@@ -1,4 +1,18 @@
 // -*- related-file-name: "../include/efont/amfm.hh" -*-
+
+/* amfm.{cc,hh} -- Adobe Multiple-Master Font Metrics
+ *
+ * Copyright (c) 1998-2003 Eddie Kohler
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; either version 2 of the License, or (at your option)
+ * any later version. This program is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
+ * Public License for more details.
+ */
+
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -271,7 +285,7 @@ AmfmReader::lwarning(const char *format, ...) const
 {
     va_list val;
     va_start(val, format);
-    _errh->verror(ErrorHandler::Warning, _l.landmark(), format, val);
+    _errh->verror(ErrorHandler::ERR_WARNING, _l.landmark(), format, val);
     va_end(val);
 }
 
@@ -280,7 +294,7 @@ AmfmReader::lerror(const char *format, ...) const
 {
     va_list val;
     va_start(val, format);
-    _errh->verror(ErrorHandler::Error, _l.landmark(), format, val);
+    _errh->verror(ErrorHandler::ERR_ERROR, _l.landmark(), format, val);
     va_end(val);
 }
 
@@ -308,7 +322,7 @@ AmfmReader::check_mmspace()
     if (!_mmspace && _amfm->_naxes >= 0 && _amfm->_nmasters >= 0
 	&& _amfm->_font_name) {
 	_mmspace = _amfm->_mmspace =
-	    new EfontMMSpace(_amfm->_font_name, _amfm->_naxes, _amfm->_nmasters);
+	    new MultipleMasterSpace(_amfm->_font_name, _amfm->_naxes, _amfm->_nmasters);
     }
 }
 
@@ -316,7 +330,7 @@ AmfmReader::check_mmspace()
 bool
 AmfmReader::read()
 {
-    assert(_amfm);
+    assert(_amfm != 0);
     _mmspace = _amfm->_mmspace;
   
     AfmParser &l = _l;
@@ -481,7 +495,7 @@ AmfmReader::read()
 	return false;
     }
   
-    PinnedErrorHandler pin_errh(_errh, _l.landmark());
+    LandmarkErrorHandler pin_errh(_errh, _l.landmark());
     if (!_amfm->sanity(&pin_errh)) {
 	_errh->lerror(_l.landmark().whole_file(),
 		      "bad AMFM file (missing or inconsistent information)");
