@@ -1,4 +1,18 @@
 // -*- related-file-name: "../include/efont/t1font.hh" -*-
+
+/* t1font.{cc,hh} -- Type 1 font
+ *
+ * Copyright (c) 1998-2003 Eddie Kohler
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; either version 2 of the License, or (at your option)
+ * any later version. This program is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
+ * Public License for more details.
+ */
+
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -56,6 +70,13 @@ Type1Font::~Type1Font()
 	    delete _glyphs[i];
 }
 
+void
+Type1Font::set_item(int i, Type1Item *it)
+{
+    delete _items[i];
+    _items[i] = it;
+}
+
 void
 Type1Font::add_definition(int dict, Type1Definition *t1d)
 {
@@ -65,6 +86,12 @@ Type1Font::add_definition(int dict, Type1Definition *t1d)
 	_index[dict] = _items.size() - 1;
 }
 
+void
+Type1Font::add_dict_size(int dict, int delta)
+{
+    _dict_deltas[dict] += delta;
+}
+
 void
 Type1Font::add_glyph(Type1Subr *s)
 {
@@ -276,38 +303,41 @@ Type1Font::ok() const
 void
 Type1Font::read_encoding(Type1Reader &reader, const char *first_line)
 {
-    while (*first_line == ' ') first_line++;
+    while (isspace(*first_line))
+	first_line++;
     if (strncmp(first_line, "StandardEncoding", 16) == 0) {
 	_encoding = Type1Encoding::standard_encoding();
 	add_item(_encoding);
 	return;
     }
-  
+
     _encoding = new Type1Encoding;
     add_item(_encoding);
   
     bool got_any = false;
     StringAccum accum;
+    
     while (reader.next_line(accum)) {
     
 	// check for NULL STRING
-	if (!accum.length()) continue;
+	if (!accum.length())
+	    continue;
 	accum.append('\0');		// ensure we don't run off the string
 	char *pos = accum.data();
     
 	// skip to first `dup' token
 	if (!got_any) {
-	    pos = strstr(pos, "dup");
-	    if (!pos) {
-		accum.clear();
-		continue;
+	    if (!(pos = strstr(pos, "dup"))) {
+		pos = accum.data();
+		goto check_done;
 	    }
 	}
     
 	// parse as many `dup INDEX */CHARNAME put' as there are in the line
 	while (1) {
 	    // skip spaces, look for `dup '
-	    while (isspace(pos[0])) pos++;
+	    while (isspace(pos[0]))
+		pos++;
 	    if (pos[0] != 'd' || pos[1] != 'u' || pos[2] != 'p' || !isspace(pos[3]))
 		break;
       
@@ -331,15 +361,17 @@ Type1Font::read_encoding(Type1Reader &reader, const char *first_line)
 	    got_any = true;
 	    pos = scan + 3;
 	}
-    
-	// add COPY ITEM if necessary for leftovers we didn't parse
-	if (got_any && *pos)
-	    add_item(new Type1CopyItem(String(pos)));
-    
+
+      check_done:
 	// check for end of encoding section
-	if (strstr(pos, "readonly") != 0 || strstr(pos, "def") != 0)
+	// if not over, add COPY ITEM for leftovers we didn't parse
+	if ((strstr(pos, "readonly") != 0 || strstr(pos, "def") != 0)
+	    && (got_any || strstr(pos, "for") == 0)) {
+	    _encoding->set_definer(String(pos));
 	    return;
-    
+	} else if (got_any && *pos)
+	    add_item(new Type1CopyItem(String(pos)));
+
 	accum.clear();
     }
 }
@@ -472,7 +504,7 @@ Type1Font::undo_synthetic()
 		oth_glyphs = subr_grp;
 	}
 
-    assert(oth_glyphs);
+    assert(oth_glyphs != 0);
   
     for (int i = nitems() - 1; i >= 0; i--)
 	if (Type1SubrGroupItem *subr_grp = _items[i]->cast_subr_group()) {
@@ -750,7 +782,7 @@ Type1Font::cache_defs() const
     _cached_defs = true;
 }
 
-EfontMMSpace *
+MultipleMasterSpace *
 Type1Font::mmspace() const
 {
     if (!_cached_mmspace)
@@ -758,7 +790,7 @@ Type1Font::mmspace() const
     return _mmspace;
 }
 
-EfontMMSpace *
+MultipleMasterSpace *
 Type1Font::create_mmspace(ErrorHandler *errh) const
 {
     if (_cached_mmspace)
@@ -778,7 +810,7 @@ Type1Font::create_mmspace(ErrorHandler *errh) const
 	return 0;
     }
     int naxes = master_positions[0].size();
-    _mmspace = new EfontMMSpace(font_name(), naxes, nmasters);
+    _mmspace = new MultipleMasterSpace(font_name(), naxes, nmasters);
     _mmspace->set_master_positions(master_positions);
   
     Vector< Vector<double> > normalize_in, normalize_out;
