@@ -1,4 +1,18 @@
 // -*- related-file-name: "../include/efont/t1item.hh" -*-
+
+/* t1item.{cc,hh} -- items in a Type 1 font
+ *
+ * Copyright (c) 1998-2003 Eddie Kohler
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; either version 2 of the License, or (at your option)
+ * any later version. This program is distributed in the hope that it will be
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
+ * Public License for more details.
+ */
+
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -584,14 +598,14 @@ static const char *standard_encoding_defs[] = {
 
 
 Type1Encoding::Type1Encoding()
-    : _v(new PermString[256]), _copy_of(0)
+    : _v(new PermString[256]), _copy_of(0), _definer("readonly def")
 {
     for (int i = 0; i < 256; i++)
 	_v[i] = dot_notdef;
 }
 
 Type1Encoding::Type1Encoding(const Type1Encoding &o)
-    : Type1Item()
+    : Type1Item(), _definer(o._definer)
 {
     if (o._copy_of) {
 	_v = o._v;
@@ -605,7 +619,7 @@ Type1Encoding::Type1Encoding(const Type1Encoding &o)
 }
 
 Type1Encoding::Type1Encoding(Type1Encoding *copy_of)
-    : _v(copy_of->_v), _copy_of(copy_of)
+    : _v(copy_of->_v), _copy_of(copy_of), _definer(copy_of->_definer)
 {
 }
 
@@ -642,6 +656,7 @@ Type1Encoding::gen(Type1Writer &w)
 	for (int i = 0; i < 256; i++)
 	    if (_v[i] != dot_notdef)
 		w << "dup " << i << " /" << _v[i] << " put\n";
+	w << _definer << '\n';
     }
 }
 
