@@ -45,7 +45,7 @@ Clp_Option options[] = {
   { "kern-precision", 'k', KERN_PREC_OPT, Clp_ArgDouble, 0 },
   { "output", 'o', OUTPUT_OPT, Clp_ArgString, 0 },
   { "precision", 'p', PRECISION_OPT, Clp_ArgInt, 0 },
-  { "version", 0, VERSION_OPT, 0, 0 },
+  { "version", 'v', VERSION_OPT, 0, 0 },
   { "help", 'h', HELP_OPT, 0, 0 },
 };
 
@@ -197,7 +197,7 @@ usage_error(char *error_message, ...)
   if (!error_message)
     errh->message("Usage: %s [OPTION | FONT]...", program_name);
   else
-    errh->verror(ErrorHandler::Error, String(), error_message, val);
+    errh->verror(ErrorHandler::ERR_ERROR, String(), error_message, val);
   errh->message("Type %s --help for more information.", program_name);
   exit(1);
 }
@@ -206,7 +206,7 @@ static void
 usage()
 {
   printf("\
-`Mmafm' creates an AFM font metrics file for a multiple master font by\n\
+'Mmafm' creates an AFM font metrics file for a multiple master font by\n\
 interpolating at a point you specify and writes it to the standard output.\n\
 \n\
 Usage: %s [OPTION | FONT]...\n\
@@ -218,7 +218,7 @@ the PSRESOURCEPATH environment variable.\n\
 General options:\n\
   -o, --output=FILE             Write output to FILE.\n\
   -h, --help                    Print this message and exit.\n\
-      --version                 Print version number and warranty and exit.\n\
+  -v, --version                 Print version number and warranty and exit.\n\
 \n\
 Interpolation settings:\n\
   -w, --weight=N                Set weight to N.\n\
@@ -247,9 +247,8 @@ main(int argc, char **argv)
     Clp_NewParser(argc, argv, sizeof(options) / sizeof(options[0]), options);
   program_name = Clp_ProgramName(clp);
 
-  errh = new FileErrorHandler(stderr);
+  errh = ErrorHandler::static_initialize(new FileErrorHandler(stderr, String(program_name) + ": "));
   
-  char *output_name = "<stdout>";
   FILE *output_file = 0;
   int precision = 3;
   double kern_precision = 2.0;
@@ -290,12 +289,12 @@ main(int argc, char **argv)
       
      case OUTPUT_OPT:
       if (output_file) errh->fatal("output file already specified");
-      if (strcmp(clp->arg, "-") == 0) {
+      if (strcmp(clp->arg, "-") == 0)
 	output_file = stdout;
-	output_name = "<stdout>";
-      } else {
+      else {
 	output_file = fopen(clp->arg, "wb");
-	if (!output_file) errh->fatal("can't open `%s' for writing", clp->arg);
+	if (!output_file)
+	    errh->fatal("%s: %s", clp->arg, strerror(errno));
       }
       break;
       
@@ -330,7 +329,7 @@ particular purpose.\n");
  done:
   if (!amfm) usage_error("missing font argument");
   
-  EfontMMSpace *mmspace = amfm->mmspace();
+  MultipleMasterSpace *mmspace = amfm->mmspace();
 #if MMAFM_RUN_MMPFB
   if (!mmspace->check_intermediate()) {
     char *buf = new char[amfm->font_name().length() + 30];
