@@ -54,7 +54,7 @@ Clp_Option options[] = {
   { "quiet", 'q', QUIET_OPT, 0, Clp_Negate },
   { "style", 0, STYLE_OPT, Clp_ArgDouble, 0 },
   { "subrs", 0, SUBRS_OPT, Clp_ArgInt, Clp_Negate },
-  { "version", 0, VERSION_OPT, 0, 0 },
+  { "version", 'v', VERSION_OPT, 0, 0 },
   { "wd", 0, WIDTH_OPT, Clp_ArgDouble, 0 },
   { "weight", 'w', WEIGHT_OPT, Clp_ArgDouble, 0 },
   { "width", 'W', WIDTH_OPT, Clp_ArgDouble, 0 },
@@ -66,7 +66,7 @@ using namespace Efont;
 static const char *program_name;
 static ErrorHandler *errh;
 static MyFont *font;
-static EfontMMSpace *mmspace;
+static MultipleMasterSpace *mmspace;
 
 static Vector<PermString> ax_names;
 static Vector<int> ax_nums;
@@ -81,7 +81,7 @@ usage_error(char *error_message, ...)
   if (!error_message)
     errh->message("Usage: %s [OPTION]... FONT", program_name);
   else
-    errh->verror(ErrorHandler::Error, String(), error_message, val);
+    errh->verror(ErrorHandler::ERR_ERROR, String(), error_message, val);
   errh->message("Type %s --help for more information.", program_name);
   exit(1);
 }
@@ -90,7 +90,7 @@ void
 usage()
 {
   printf("\
-`Mmpfb' creates a single-master PostScript Type 1 font by interpolating a\n\
+'Mmpfb' creates a single-master PostScript Type 1 font by interpolating a\n\
 multiple master font at a point you specify. The resulting font does not\n\
 contain multiple master extensions. It is written to the standard output.\n\
 \n\
@@ -107,11 +107,10 @@ General options:\n\
   -o, --output=FILE            Write output to FILE.\n\
   -p, --precision=N            Set precision to N (larger means more precise).\n\
       --subrs=N                Limit output font to at most N subroutines.\n\
-  -m, --minimize               Replace unusual PostScript in font definition\n\
-                               with well-known code.\n\
+      --no-minimize            Do not replace original font's PostScript code.\n\
   -h, --help                   Print this message and exit.\n\
   -q, --quiet                  Do not generate any error messages.\n\
-      --version                Print version number and exit.\n\
+  -v, --version                Print version number and exit.\n\
 \n\
 Interpolation settings:\n\
   -w, --weight=N               Set weight to N.\n\
@@ -182,6 +181,10 @@ do_file(const char *filename, PsresDatabase *psres)
   
   font = new MyFont(*reader);
   delete reader;
+  if (!font->ok())
+      errh->fatal("%s: invalid font", filename);
+  else if (font->nglyphs() == 0)
+      errh->fatal("%s: font contains no characters", filename);
   
   mmspace = font->create_mmspace(errh);
   if (!mmspace)
@@ -212,7 +215,7 @@ print_conversion_program(FILE *f, const Type1Charstring &cs, PermString name)
 
 
 static void
-print_amcp_info(EfontMMSpace *mmspace, FILE *f)
+print_amcp_info(MultipleMasterSpace *mmspace, FILE *f)
 {
   const Type1Charstring &ndv = mmspace->ndv();
   const Type1Charstring &cdv = mmspace->cdv();
@@ -241,13 +244,12 @@ main(int argc, char **argv)
   
   bool write_pfb = true;
   bool amcp_info = false;
-  bool minimize = false;
+  bool minimize = true;
   int precision = 5;
   int subr_count = -1;
   FILE *outfile = 0;
-  ErrorHandler *default_errh =
-    new PinnedErrorHandler(new FileErrorHandler(stderr), "mmpfb");
-  errh = default_errh;
+  ::errh =
+      ErrorHandler::static_initialize(new FileErrorHandler(stderr, String(program_name) + ": "));
   
   while (1) {
     int opt = Clp_Next(clp);
@@ -314,7 +316,7 @@ main(int argc, char **argv)
       
      case QUIET_OPT:
       if (clp->negated)
-	errh = default_errh;
+	errh = ErrorHandler::default_handler();
       else
 	errh = ErrorHandler::silent_handler();
       break;
@@ -325,7 +327,8 @@ main(int argc, char **argv)
 	outfile = stdout;
       else {
 	outfile = fopen(clp->arg, "wb");
-	if (!outfile) errh->fatal("can't open `%s' for writing", clp->arg);
+	if (!outfile)
+	    errh->fatal("%s: %s", clp->arg, strerror(errno));
       }
       break;
       
@@ -348,7 +351,8 @@ particular purpose.\n");
       break;
       
      case Clp_Done:
-      if (!font) usage_error("missing font argument");
+      if (!font)
+	  usage_error("missing font argument");
       goto done;
       
      case Clp_BadOption:
@@ -362,7 +366,8 @@ particular purpose.\n");
   }
   
  done:
-  if (outfile == 0) outfile = stdout;
+  if (outfile == 0)
+      outfile = stdout;
   
   if (amcp_info) {
     print_amcp_info(mmspace, outfile);
@@ -387,7 +392,7 @@ particular purpose.\n");
   if (!font->set_design_vector(mmspace, design, errh))
     exit(1);
 
-  font->interpolate_dicts(errh);
+  font->interpolate_dicts(minimize, errh);
   font->interpolate_charstrings(precision, errh);
 
   if (subr_count >= 0) {
