@@ -3,6 +3,7 @@
 #endif
 #include "t1rewrit.hh"
 #include <efont/t1item.hh>
+#include <efont/t1csgen.hh>
 #include <lcdf/error.hh>
 #include <lcdf/straccum.hh>
 #include <cstdio>
@@ -13,29 +14,6 @@ using namespace Efont;
 static bool itc_complained = false;
 static ErrorHandler *itc_errh;
 
-static const char * const command_desc[] = {
-    0, 0, 0, 0, "y",
-    "xy", "x", "y", "xyxyxy", 0,
-
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-
-    0, "xy", "x", 0, 0,
-    0, 0, 0, 0, 0,
-
-    "yxyx", "xxyy", 0, 0, 0,
-    0, 0, 0, 0, 0,
-
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-  
-    0, 0, 0, 0, 0,
-    0, 0, 0, 0, 0,
-  
-    0, 0, 0, 0, 0,
-    "XY", 0, 0, 0, 0
-};
-
 void
 itc_complain()
 {
@@ -44,137 +22,14 @@ itc_complain()
 }
 
 
-Type1CharstringGen::Type1CharstringGen(int precision)
-{
-    if (precision >= 1 && precision <= 107)
-	_precision = precision;
-    else
-	_precision = 5;
-    _f_precision = _precision;
-}
-
-void
-Type1CharstringGen::clear()
-{
-    _ncs.clear();
-    _true_x = _true_y = _false_x = _false_y = 0;
-}
-
-void
-Type1CharstringGen::gen_number(double float_val, int kind)
-{
-    switch (kind) {
-      case 'x':
-	_true_x += float_val;
-	float_val = _true_x - _false_x;
-	break;
-      case 'y':
-	_true_y += float_val;
-	float_val = _true_y - _false_y;
-	break;
-      case 'X':
-	_true_x = float_val;
-	break;
-      case 'Y':
-	_true_y = float_val;
-	break;
-    }
-    
-    int big_val = (int)floor(float_val * _f_precision + 0.5);
-    int val = big_val / _precision;
-    int frac = big_val % _precision;
-    if (frac != 0)
-	val = big_val;
-
-    if (val >= -107 && val <= 107)
-	_ncs.append((char)(val + 139));
-
-    else if (val >= -1131 && val <= 1131) {
-	int base = val < 0 ? 251 : 247;
-	if (val < 0) val = -val;
-	val -= 108;
-	int w = val % 256;
-	val = (val - w) / 256;
-	_ncs.append((char)(val + base));
-	_ncs.append((char)w);
-
-    } else {
-	_ncs.append('\377');
-	long l = val;
-	_ncs.append((char)((l >> 24) & 0xFF));
-	_ncs.append((char)((l >> 16) & 0xFF));
-	_ncs.append((char)((l >> 8) & 0xFF));
-	_ncs.append((char)((l >> 0) & 0xFF));
-    }
-
-    if (frac != 0) {
-	_ncs.append((char)(_precision + 139));
-	_ncs.append((char)Charstring::cEscape);
-	_ncs.append((char)(Charstring::cDiv - Charstring::cEscapeDelta));
-    }
-
-    float_val = (double)big_val / _precision;
-    switch (kind) {
-      case 'x':
-	_false_x += float_val;
-	break;
-      case 'y':
-	_false_y += float_val;
-	break;
-      case 'X':
-	_false_x = float_val;
-	break;
-      case 'Y':
-	_false_y = float_val;
-	break;
-    }
-}
-
-
-void
-Type1CharstringGen::gen_command(int command)
-{
-    if (command >= Charstring::cEscapeDelta) {
-	_ncs.append((char)Charstring::cEscape);
-	_ncs.append((char)(command - Charstring::cEscapeDelta));
-    } else
-	_ncs.append((char)command);
-}
-
-void
-Type1CharstringGen::gen_stack(CharstringInterp &interp, int for_cmd)
-{
-    const char *str = (for_cmd <= Charstring::cLastCommand ? command_desc[for_cmd] : (const char *)0);
-    int i;
-    for (i = 0; str && *str && i < interp.size(); i++, str++)
-	gen_number(interp.at(i), *str);
-    for (; i < interp.size(); i++)
-	gen_number(interp.at(i));
-    interp.clear();
-}
-
-
-Type1Charstring *
-Type1CharstringGen::output()
-{
-    return new Type1Charstring(_ncs.take_string());
-}
-
-void
-Type1CharstringGen::output(Type1Charstring &cs)
-{
-    cs.assign(_ncs.take_string());
-}
-
-
-
 /*****
  * HintReplacementDetector
  **/
 
 class HintReplacementDetector : public CharstringInterp { public:
 
-    HintReplacementDetector(Type1Font *, Vector<double> *, int);
+    HintReplacementDetector(Type1Font *, int);
+    HintReplacementDetector(Type1Font *, const Vector<double> &, int);
 
     bool is_hint_replacement(int i) const	{ return _hint_replacements[i]; }
     int call_count(int i) const		{ return _call_counts[i]; }
@@ -193,7 +48,14 @@ class HintReplacementDetector : public CharstringInterp { public:
 
 };
 
-HintReplacementDetector::HintReplacementDetector(Type1Font *f, Vector<double> *wv, int b)
+HintReplacementDetector::HintReplacementDetector(Type1Font *f, int b)
+    : CharstringInterp(f),
+      _hint_replacements(f->nsubrs(), 0), _call_counts(f->nsubrs(), 0),
+      _count_calls_below(b)
+{
+}
+
+HintReplacementDetector::HintReplacementDetector(Type1Font *f, const Vector<double> &wv, int b)
     : CharstringInterp(f, wv),
       _hint_replacements(f->nsubrs(), 0), _call_counts(f->nsubrs(), 0),
       _count_calls_below(b)
@@ -391,8 +253,8 @@ Type1OneMMRemover::run_subr(Type1Charstring *cs)
 bool
 Type1OneMMRemover::itc_command(int command, int on_stack)
 {
-    Vector<double> *weight = weight_vector();
-    assert(weight);
+    const Vector<double> &weight = weight_vector();
+    assert(weight.size());
     Vector<double> *scratch = scratch_vector();
     Type1CharstringGen *gen =
 	(_in_prefix ? &_prefix_gen : (_in_subr ? &_main_gen : 0));
@@ -404,8 +266,8 @@ Type1OneMMRemover::itc_command(int command, int on_stack)
 	  if (on_stack != 1)
 	      return false;
 	  int offset = (int)at(base);
-	  for (int i = 0; i < weight->size(); i++)
-	      vec(scratch, offset+i) = weight->at_u(i);
+	  for (int i = 0; i < weight.size(); i++)
+	      vec(scratch, offset+i) = weight.at_u(i);
 	  // save load command, so we expand its effects into the scratch
 	  // vector
 	  if (gen) {
@@ -708,7 +570,7 @@ Type1BadCallRemover::run(Type1Charstring &cs)
  * Type1MMRemover
  **/
 
-Type1MMRemover::Type1MMRemover(Type1Font *font, Vector<double> *wv,
+Type1MMRemover::Type1MMRemover(Type1Font *font, const Vector<double> &wv,
 			       int precision, ErrorHandler *errh)
     : _font(font), _weight_vector(wv), _precision(precision),
       _nsubrs(font->nsubrs()),
@@ -730,7 +592,8 @@ Type1MMRemover::Type1MMRemover(Type1Font *font, Vector<double> *wv,
 	    _hint_replacement_subr[i] = 1;
 
     // don't get rid of first 4 subrs
-    _subr_done[0] = _subr_done[1] = _subr_done[2] = _subr_done[3] = 1;
+    for (int i = 0; i < _nsubrs && i < 4; i++)
+	_subr_done[i] = 1;
 }
 
 Type1MMRemover::~Type1MMRemover()
@@ -744,14 +607,16 @@ Type1MMRemover::~Type1MMRemover()
 Type1Charstring *
 Type1MMRemover::subr_prefix(int subrno)
 {
-    if (subrno < 0 || subrno >= _nsubrs) return 0;
+    if (subrno < 0 || subrno >= _nsubrs)
+	return 0;
   
     if (!_subr_done[subrno]) {
 	_subr_done[subrno] = 1;
     
 	Type1Charstring *subr = _font->subr(subrno);
-	if (!subr) return 0;
-    
+	if (!subr)
+	    return 0;
+
 	Type1OneMMRemover one(this);
 	if (one.run_fresh_subr(*subr, !_hint_replacement_subr[subrno]))
 	    _must_expand_subr[subrno] = true;
@@ -790,8 +655,8 @@ Type1MMRemover::run()
     Type1OneMMRemover one(this);
   
     // check subroutines
-    for (int i = 0; i < _nsubrs; i++)
-	(void)subr_prefix(i);
+    for (int subrno = 0; subrno < _nsubrs; subrno++)
+	(void)subr_prefix(subrno);
 
     // expand glyphs
     Vector<PermString> bad_glyphs;
@@ -821,29 +686,28 @@ Type1MMRemover::run()
 	if (Type1Subr *g = _font->glyph_x(i))
 	    hr.run(g->t1cs());
     // don't remove first four subroutines!
-    for (int i = 4; i < _nsubrs; i++)
-	if (hr.call_count(i) || _hint_replacement_subr[i]) {
-	    Type1Charstring *cs = _font->subr(i);
+    for (int subrno = 4; subrno < _nsubrs; subrno++)
+	if (hr.call_count(subrno) || _hint_replacement_subr[subrno]) {
+	    Type1Charstring *cs = _font->subr(subrno);
 	    if (one.rerun_subr(*cs)) {
 		_expand_all_subrs = true;
 		if (one.rerun_subr(*cs))
-		    bad_glyphs.push_back(permprintf("subr %d", i));
+		    bad_glyphs.push_back(permprintf("subr %d", subrno));
 		_expand_all_subrs = false;
 	    }
 	    one.output_main(*cs);
 	} else
-	    _font->remove_subr(i);
+	    _font->remove_subr(subrno);
   
     // remove calls to removed subroutines
     Type1BadCallRemover bcr(this);
     for (int i = 0; i < _font->nglyphs(); i++)
 	if (Type1Subr *g = _font->glyph_x(i))
 	    bcr.run(g->t1cs());
-    for (int i = 4; i < _nsubrs; i++)
-	if (Type1Charstring *cs = _font->subr(i))
+    for (int subrno = 4; subrno < _nsubrs; subrno++)
+	if (Type1Charstring *cs = _font->subr(subrno))
 	    bcr.run(*cs);
   
-  
     // report warnings
     if (bad_glyphs.size()) {
 	qsort(&bad_glyphs[0], bad_glyphs.size(), sizeof(PermString), sort_permstring_compare);
@@ -887,7 +751,7 @@ class SubrExpander : public CharstringInterp { public:
 };
 
 SubrExpander::SubrExpander(Type1Font *font)
-    : CharstringInterp(font, 0), _gen(0), _renumbering(0)
+    : CharstringInterp(font), _gen(0), _renumbering(0)
 {
 }
 
@@ -906,14 +770,14 @@ SubrExpander::type1_command(int cmd)
       case CS::cCallsubr: {
 	  if (size() < 1)
 	      goto unknown;
-	  int which = (int)top(0);
-	  int renumber_which = (which >= 0 && which < _renumbering->size() ? (*_renumbering)[which] : which);
-	  if (renumber_which >= 0) {
-	      top(0) = renumber_which;
+	  int subrno = (int)top(0);
+	  int renumber_subrno = (subrno >= 0 && subrno < _renumbering->size() ? (*_renumbering)[subrno] : subrno);
+	  if (renumber_subrno >= 0) {
+	      top(0) = renumber_subrno;
 	      goto unknown;
 	  }
 	  pop();
-	  if (Charstring *subr_cs = get_subr(which)) {
+	  if (Charstring *subr_cs = get_subr(subrno)) {
 	      _subr_level++;
 	      subr_cs->run(*this);
 	      _subr_level--;
@@ -966,7 +830,7 @@ Type1SubrRemover::Type1SubrRemover(Type1Font *font, ErrorHandler *errh)
       _save_count(0), _nonexist_count(0), _errh(errh)
 {
     // find subroutines needed for hint replacement
-    HintReplacementDetector hr(font, 0, 2);
+    HintReplacementDetector hr(font, 2);
     for (int i = 0; i < _font->nglyphs(); i++) {
 	Type1Subr *g = _font->glyph_x(i);
 	if (g)
